Package: clustMixType
Version: 0.5-2
Date: 2026-06-25
Title: k-Prototypes Clustering for Mixed Variable-Type Data
Authors@R: c(person(given = "Gero", 
 		    family = "Szepannek", 
                    role = c("aut", "cre"), 
                    email = "gero.szepannek@web.de"),
             person(given = "Rabea", 
                    family = "Aschenbruck", 
                    role = "aut")
             )
Imports: RColorBrewer, tibble, combinat, dplyr, rlang, cluster, mclust
Suggests: testthat
Description: Functions to perform k-prototypes partitioning clustering for
    mixed variable-type data according to Z.Huang (1998): Extensions to the k-Means
    Algorithm for Clustering Large Data Sets with Categorical Variables, Data Mining
    and Knowledge Discovery 2, 283-304 and G.Szepannek et al. (2025): Clustering 
    Large Mixed-Type Data with Ordinal Variables, Adv Data Anal Classif 19, 749–767.
License: GPL (>= 2)
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-06-28 10:19:21 UTC; szepannek
Encoding: UTF-8
Author: Gero Szepannek [aut, cre],
  Rabea Aschenbruck [aut]
Maintainer: Gero Szepannek <gero.szepannek@web.de>
Repository: CRAN
Date/Publication: 2026-06-29 06:50:13 UTC
Built: R 4.6.0; ; 2026-06-29 08:29:54 UTC; unix
