# Generated by roxygen2: do not edit by hand

S3method(kproto,default)
S3method(plot,kproto)
S3method(predict,kproto)
S3method(print,kproto)
S3method(summary,kproto)
export(clprofiles)
export(importance_kproto)
export(kproto)
export(kproto2silhouette)
export(lambdaest)
export(stability_kproto)
export(validation_kproto)
importFrom(RColorBrewer,brewer.pal)
importFrom(cluster,silhouette)
importFrom(combinat,permn)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,lag)
importFrom(dplyr,lead)
importFrom(dplyr,mutate)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(mclust,adjustedRandIndex)
importFrom(rlang,.data)
importFrom(stats,complete.cases)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,qgeom)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(tibble,is_tibble)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,tail)
