# Development version of the R Package clustMixType

<img src="man/figures/logo.png" align="right" height="139" />

k-Prototypes Clustering for Mixed Variable-Type Data. For details cf. the [R Journal paper](https://journal.r-project.org/archive/2018/RJ-2018-048/index.html).

Corresponding papers on [valdiation indices](https://publikationen.bibliothek.kit.edu/1000120412) 
and [missing data](https://link.springer.com/article/10.1007/s00357-022-09422-y) for k prototypes.


## Installation

Install the current CRAN release of the package in R:

``` r
install.packages(clustMixType)
```

Install the latest development version of the package in R from this repository.
``` r
remotes::install_github("https://github.com/g-rho/clustMixType.git")
```
